#!/usr/bin/perl -w

#
# Interface to perl-Bootloader library
#

package Bootloader_API;

use strict;

use Bootloader::Library;
use LIMAL::LogHelper;

our %TYPEINFO;

BEGIN { $TYPEINFO{ALL_METHODS} = 0; }

# create a new bootloader object
my $lib_ref = Bootloader::Library->new();

my %lines_cache = ();


# Log collected log messages
sub DumpLog {
    foreach my $rec (@{$lib_ref->GetLogRecords()})
    {
	if ($rec->{"level"} eq "debug")
	{
	    LIMAL::LogHelper::logDebug ($rec->{"message"});
	}
	elsif ($rec->{"level"} eq "milestone")
	{
	    LIMAL::LogHelper::logInfo ($rec->{"message"});
	}
	elsif ($rec->{"level"} eq "warning")
	{
	    LIMAL::LogHelper::logError ("WARNING: " . $rec->{"message"});
	}
	elsif ($rec->{"level"} eq "error")
	{
	    LIMAL::LogHelper::logError ($rec->{"message"});
	}
	else
	{
	    LIMAL::LogHelper::logError ("Incomplete log record");
	    LIMAL::LogHelper::logError ($rec->{"message"});
	}
    }
}

BEGIN { $TYPEINFO{initBootloader} = ["function", "void", "string"]; }
# library initialization
sub initBootloader($) {
    my ($lt) = @_;
    my $ret = $lib_ref->Initialize($lt);

    %lines_cache = ();
    DumpLog();
    return $ret;
}

BEGIN { $TYPEINFO{updateBootloader} = ["function", "boolean", "boolean"]; }
sub updateBootloader() {
    my ($avoid_init) = @_;
    my $ret = $lib_ref->UpdateBootloader($avoid_init);

    DumpLog();
    return $ret;
}

BEGIN { $TYPEINFO{initializeBootloader} = ["function", "boolean"]; }
sub initializeBootloader() {
    my $ret = $lib_ref->InitializeBootloader();

    DumpLog();
    return $ret;
}

BEGIN { $TYPEINFO{initConfig} = ["function", "boolean"]; }
# read configuration
sub initConfig() {
    my $ret = $lib_ref->ReadSettings();

    DumpLog();
    if (!$ret)
    {
	return "false";
    }
    return "true";
}

BEGIN { $TYPEINFO{getDeviceMapping} = ["function", ["map", "string", "string"]]; }
sub getDeviceMapping() {
    my $devmap = $lib_ref->GetDeviceMapping () || {};
    DumpLog();
    return $devmap;
}

BEGIN { $TYPEINFO{getGlobalOptions} = ["function", ["map", "string", "string"]]; }
sub getGlobalOptions() {
    my %globalsettings = %{$lib_ref->GetGlobalSettings () || {}};

    # remove "__lines" key - it's internal
    $globalsettings{"lines_cache_id"} = 0;
    $lines_cache{0} = $globalsettings{'__lines'};
    delete $globalsettings{'__lines'};

    # if there is "stage1_dev" key then convert the values to single string
    if (defined $globalsettings{'stage1_dev'})
    {
	my @list = @{$globalsettings{'stage1_dev'}};

	my $string = '';
	my $first = 1;

	foreach my $dev (@list) {
	    if ($first == 1)
	    {
		$first = 0;
	    }
	    else
	    {
		$string = $string.',';
	    }

	    $string = $string.$dev;
	}

	$globalsettings{'stage1_dev'} = $string;
    }
   
    my %ret;
    # convert data to string type
    while ((my $key, my $value) = each %globalsettings) {
	$ret{"$key"} = "$value";
    }

    return \%ret; 
}

BEGIN { $TYPEINFO{commitSettings} = ["function", "boolean"]; }
# write settings to the files
sub commitSettings () {
    my $ret = $lib_ref->WriteSettings();
    DumpLog();

    return $ret;
}

BEGIN { $TYPEINFO{setDeviceMapping} = ["function", "boolean", ["map", "string", "string"]]; }
sub setDeviceMapping($) {
    my ($dm) = @_;

    my $ret = $lib_ref->SetDeviceMapping ($dm);
    DumpLog();
    return $ret;
}

BEGIN { $TYPEINFO{setMountPoints} = ["function", "boolean", ["map", "string", "string"]]; }
sub setMountPoints($) {
    my ($dm) = @_;
    my $ret = $lib_ref->DefineMountPoints ($dm);

    DumpLog();
    return $ret; 
}

BEGIN { $TYPEINFO{setPartitions} = ["function", "boolean", ["list", ["list", "string"]]]; }
sub setPartitions($) {
    my ($dm) = @_;
    my $ret = $lib_ref->DefinePartitions($dm);

    DumpLog();
    return $ret; 
}

BEGIN { $TYPEINFO{setMDArrays} = ["function", "boolean", ["map", "string", ["list", "string"]]]; }
sub setMDArrays($) {
    my ($dm) = @_;
    my $ret = $lib_ref->DefineMDArrays($dm);

    DumpLog();
    return $ret; 
}

BEGIN { $TYPEINFO{setFilesFromStrings} = ["function", "boolean", ["map", "string", "string"]]; }
sub setFilesFromStrings($) {
    my ($contents) = @_;

    my $ret = $lib_ref->SetFilesContents($contents);

    DumpLog();
    return $ret; 
}

BEGIN { $TYPEINFO{getFilesContents} = ["function", ["map", "string", "string"]]; }
sub getFilesContents() {
    my $ret = $lib_ref->GetFilesContents();
    
    DumpLog();
    return $ret; 
}

BEGIN { $TYPEINFO{setGlobalOptions} = ["function", "boolean", ["map", "string", "string"]]; }
sub setGlobalOptions($) {
    my ($globals) = @_;
    my %globalhash = %{$globals};

    # split device string into a list
    my @devices = split(',', $globalhash{'stage1_dev'});
    $globalhash{'stage1_dev'} = \@devices;

    my $index = $globalhash{"lines_cache_id"};
    if ($index != undef)
    {
	$globalhash{"__lines"} = $lines_cache{$index};
    }

    my $ret = $lib_ref->SetGlobalSettings (\%globalhash);
    DumpLog();
    return $ret;
}

BEGIN { $TYPEINFO{getSections} = ["function", ["list", ["map", "string", "string"]]]; }
sub getSections() {
    my @sections = @{$lib_ref->GetSections () || []};

    my $index = -1;
    # remove "__lines" key - it's internal
    foreach my $section (@sections) { 
	$index = $index + 1;
	$lines_cache{$index} = $section->{'__lines'};
	delete $section->{'__lines'};
	$section->{"lines_cache_id"} = $index;
    }
    DumpLog();
    return \@sections;
}

BEGIN { $TYPEINFO{setSections} = ["function", "boolean", ["list", ["map", "string", "string"]]]; }
sub setSections($) {
    my ($sections) = @_;

    my @sections = @{$sections || []};
    foreach my $section (@sections) {
	my $index = $section->{"lines_cache_id"};
	if ($index != undef)
	{
	    $section->{"__lines"} = $lines_cache{$index};
	}
    }
    my $ret = $lib_ref->SetSections($sections);
    DumpLog();
    return $ret;
}

# import fails if we cannot create the object
$lib_ref;
